/*
 * $Id: Chmap.java,v 1.4 2002/08/27 15:18:47 kenta Exp $
 *
 * Copyright 2002 Kenta Cho. All rights reserved.
 */
package jp.gr.java_conf.abagames.chmap;

import org.debian.sen.*;
import org.debian.sen.ja.*;
import java.io.*;
import java.util.*;

/**
 * Chmap bootstrap.
 *
 * @author cs8k-cyu@asahi-net.or.jp
 * @version $Revision: 1.4 $
 */
public class Chmap {
  private StringTagger analyzer;

  public void initDic() throws IOException {
    System.out.println("now loading dictionary...");
    analyzer = StringTagger.getInstance(Locale.JAPANESE);
  }

  private final String DATA_DIR = "data";

  public void getPages(String topPageURL) throws IOException {
    WebPage page = new WebPage(topPageURL);
    String base = page.getBase();
    Vector anc = page.getAnchors();
    Enumeration elms = anc.elements();
    int cnt = 0;
    while ( elms.hasMoreElements() ) {
      String url = (String)elms.nextElement();
      if ( url.endsWith("/l50") ) {
        url = url.substring(0, url.length()-4);
        System.out.println(base+url);
        String fileName = DATA_DIR + File.separator + url;
        BufferedWriter writer = new BufferedWriter(new FileWriter(fileName));
        try {
          for ( int i=0 ; i<10 ; i++ ) {
            WebPage sp = new WebPage(base+url+"/"+(i*100+1)+"-"+(i*100+100));
            if ( i > 0 ) {
              sp.removeTillDd();
              if ( !sp.removeTillDd() ) break;
            }
            writer.write(sp.getContents());
          }
        } catch ( Exception e ) {
          writer.close();
          new File(fileName).delete();
          continue;
        }
        writer.close();
      }
      cnt++; if ( cnt >= 100 ) break;
    }
  }

  public void clearDataDir() throws IOException {
    File dir = new File(DATA_DIR);
    if ( dir == null ) return;
    File[] files = dir.listFiles();
    boardTable = new Hashtable();
    for ( int i=0 ; i<files.length ; i++ ) {
      files[i].delete();
    }
  }

  private char[] invalicChar =
   {'(',')','i','j','=','.','/','<','>','','','!','?',',','_','c'};

  private boolean checkInvalidTerm(String term) {
    for ( int i=0 ; i<invalicChar.length ; i++ ) {
      if ( term.indexOf(invalicChar[i]) != -1 ) return true;
    }
    return false;
  }

  private Hashtable boardTable;
  private Hashtable termTable;

  private void addTerm(String term) {
    term.trim();
    if ( checkInvalidTerm(term) ) return;
    Integer cnt = (Integer)termTable.get(term);
    if ( cnt == null ) {
      cnt = new Integer(1);
    } else {
      cnt = new Integer(cnt.intValue() + 1);
    }
    termTable.put(term, cnt);
  }

  private void checkTerms(WebPage page, int id) {
    termTable = new Hashtable();
    Vector ress;
    ress = page.getRess();
    Enumeration elms = ress.elements();
    while ( elms.hasMoreElements() ) {
      String res = (String)elms.nextElement();
      Token[] tokens;
      try {
        tokens = analyzer.analyze(res);
      } catch ( Exception e ) {
        continue;
      }
      boolean addStart = false, includeNull = false;
      StringBuffer term = null;
      if ( tokens == null ) continue;
      for ( int i=0 ; i<tokens.length ; i++ ) {
        Token token = tokens[i];
        String tkn = token.toString();
        String type = token.getPos();
        if ( !addStart ) {
          if ( type.equals("m") ) {
            if ( tkn != null ) {
              addStart = true;
              includeNull = true;
              term = new StringBuffer(tkn);
            }
          } else if ( type.startsWith("") ) {
            addStart = true;
            includeNull = false;
            term = new StringBuffer(tkn);
          }
        } else {
          if ( type.equals("m") ) {
            if ( tkn == null ) {
              addStart = false;
              includeNull = true;
              addTerm(term.toString());
            } else {
              if ( includeNull ) {
                term.append(tkn);
              }
            }
          } else if ( type.startsWith("") ) {
            term.append(tkn);
          } else if ( type.startsWith("L-") ) {
            term.append(" ");
          } else {
            addStart = false;
            if ( includeNull ) {
              addTerm(term.toString());
            }
          }
        }
      }
    }
    elms = termTable.keys();
    while ( elms.hasMoreElements() ) {
      String term = (String)elms.nextElement();
      int c = ((Integer)termTable.get(term)).intValue();
      if ( c > 1 && term.length() > 1 ) {
        Hashtable cntTable = (Hashtable)boardTable.get(term);
        if ( cntTable == null ) {
          cntTable = new Hashtable();
        }
        cntTable.put(new Integer(id), new Integer(c));
        boardTable.put(term, cntTable);
      }
    }
  }

  private Hashtable nodes = new Hashtable();

  public void getRess() throws IOException {
    File dir = new File(DATA_DIR);
    File[] files = dir.listFiles();
    boardTable = new Hashtable();
    for ( int i=0 ; i<files.length ; i++ ) {
      BufferedReader reader = new BufferedReader(new FileReader(files[i]));
      StringBuffer contents = new StringBuffer();
      for ( ; ; ) {
        String ls = reader.readLine();
        if ( ls == null ) break;
        contents.append(ls);
      }
      WebPage page = new WebPage();
      page.setContents(contents.toString());
      ChmapNode node = new ChmapNode();
      node.thread = true;
      node.branch = new Vector();
      node.word = page.getTitle();
      nodes.put(new Integer(i), node);
      checkTerms(page, i);
    }
    Enumeration elms = boardTable.keys();
    while ( elms.hasMoreElements() ) {
      String term = (String)elms.nextElement();
      Hashtable cntTable = (Hashtable)boardTable.get(term);
      if ( cntTable.size() > 1 ) {
        int ttl = 0;
        Enumeration brds = cntTable.keys();
        while ( brds.hasMoreElements() ) {
          Integer id = (Integer)brds.nextElement();
          ttl += ((Integer)cntTable.get(id)).intValue();
        }
        int bn = cntTable.size();
        float avg = (float)ttl/bn;
        if ( avg < 30.0f && ttl < 300 ) {
          ChmapNode node = new ChmapNode();
          node.thread = false;
          node.word = term;
          node.cnt = ttl; node.avg = avg;
          nodes.put(term, node);
          System.out.println(term + ": " + ttl + "/" + bn + "/" + avg);
        }
      }
    }
  }

  private final float[] avgTable = {0, 1.0f, 2.0f, 3.0f, 4.0f, 5.0f, 6.0f};
  private final int[] cntTable = {0, 3, 5, 10, 15, 20, 30};

  private final int PROPER_TERM_NUM = 64;

  public void filterNodes() {
    int[] tnNum = new int[avgTable.length];
    Enumeration keys = nodes.keys();
    while ( keys.hasMoreElements() ) {
      Object key = keys.nextElement();
      if ( key instanceof Integer ) continue;
      ChmapNode node = (ChmapNode)nodes.get(key);
      for ( int i=0 ; i<avgTable.length ; i++ ) {
        if ( node.avg < avgTable[i] || node.cnt < cntTable[i] ) break;
        tnNum[i]++;
      }
    }
    int fc = -1, foMin = 99999;
    for ( int i=0 ; i<avgTable.length ; i++ ) {
      int os = Math.abs(PROPER_TERM_NUM-tnNum[i]);
      if ( os < foMin ) {
        foMin = os;
        fc = i;
      }
    }
    keys = nodes.keys();
    while ( keys.hasMoreElements() ) {
      Object key = keys.nextElement();
      if ( key instanceof Integer ) continue;
      ChmapNode node = (ChmapNode)nodes.get(key);
      if ( node.avg < avgTable[fc] || node.cnt < cntTable[fc] ) {
        nodes.remove(key);
      } else {
        Hashtable cntTable = (Hashtable)boardTable.get(key);
        Enumeration brds = cntTable.keys();
        while ( brds.hasMoreElements() ) {
          Integer id = (Integer)brds.nextElement();
          ChmapNode prt = (ChmapNode)nodes.get(id);
          prt.branch.addElement(node);
          if ( node.branch == null ) node.branch = new Vector();
          node.branch.addElement(prt);
        }
      }
    }
  }

  private final String NODES_FILE = "chmap_nodes.dat";

  public void saveNodes() throws FileNotFoundException, IOException {
    ObjectOutputStream os = new ObjectOutputStream(
     new FileOutputStream(NODES_FILE));
    os.writeObject(nodes);
    os.close();
  }

  public void loadNodes()
   throws ClassNotFoundException, FileNotFoundException, IOException {
    ObjectInputStream is = new ObjectInputStream(
     new FileInputStream(NODES_FILE));
    nodes = (Hashtable)is.readObject();
    is.close();
  }

  public void showNodes() {
    new ChmapFrame().showChmapNodes(nodes);
  }

  public void drawNodes(String fileName) throws
   FileNotFoundException, IOException {
    new ChmapImageDrawer().drawChmapNodes(nodes, fileName);
  }

  private static void usage() {
    System.err.println(
     "Usage: chmap [-d] [-o name] URL");
  }

  private static String baseUrl;
  private static String imageFileName = null;
  private static boolean drawOnly = false;
  private static boolean analyzeOnly = false;

  private static void parseArgs(String[] args) throws IOException {
    for ( int i=0 ; i<args.length ; i++ ) {
      if ( args[i].equals("-d") ) {
        drawOnly = true;
      } else if ( args[i].equals("-a") ) {
        analyzeOnly = true;
      } else if ( args[i].equals("-o") ) {
        imageFileName = args[i+1]; i++;
      } else if ( args[i].startsWith("-") )  {
        throw new IOException("Invalid args.");
      } else {
        baseUrl = args[i];
      }
    }
  }

  public static void main(String[] args) {
    try {
      parseArgs(args);
    } catch ( Exception e ) {
      usage();
      return;
    }
    Chmap chmap = new Chmap();
    try {
      if ( !drawOnly ) {
        if ( !analyzeOnly ) {
          chmap.clearDataDir();
          chmap.getPages(baseUrl);
        }
        chmap.initDic();
        chmap.getRess();
        chmap.filterNodes();
        chmap.saveNodes();
      } else {
        chmap.loadNodes();
      }
      if ( imageFileName == null ) {
        chmap.showNodes();
      } else {
        chmap.drawNodes(imageFileName);
      }
    } catch ( Exception e ) {
      e.printStackTrace();
    }
  }
}