/*
 * $Id: ChmapDrawer.java,v 1.2 2002/07/24 13:50:18 kenta Exp $
 *
 * Copyright 2002 Kenta Cho. All rights reserved.
 */
package jp.gr.java_conf.abagames.chmap;

import javax.swing.*;
import javax.swing.text.*;
import java.awt.*;
import java.awt.geom.*;
import java.util.*;

/**
 * Chmap graph drawer.
 *
 * @author cs8k-cyu@asahi-net.or.jp
 * @version $Revision: 1.2 $
 */
public class ChmapDrawer {

  private int scX, scY, sczX, sczY;
  private Color trGreen  = new Color(0xa0, 0xff, 0xa0);
  private Color trBlue   = new Color(0xa0, 0xa0, 0xff);
  private Color trBlack  = new Color(0, 0, 0);
  private Color trSilver = new Color(0x99, 0x99, 0x99);
  private Color trWhite  = new Color(0xff, 0xff, 0xff);

  private final int KEYWORD_MAX_LGT = 10;

  public Vector nodePoints;
  public int nxMin, nxMax, nyMin, nyMax;

  public void setNodePoints(Vector nodePoints) {
    this.nodePoints = nodePoints;
  }

  public void setNodePointsRange(int nxMin, int nxMax, int nyMin, int nyMax) {
    this.nxMin = nxMin; this.nxMax = nxMax;
    this.nyMin = nyMin; this.nyMax = nyMax;
  }

  public void showNodes(Graphics grp, Dimension dim) {
    Graphics2D g = (Graphics2D)grp;
    if ( g == null ) return;
    if ( nodePoints == null ) return;

    sczX = (int)(256*dim.getWidth()/(nxMax-nxMin));
    sczY = (int)(256*dim.getHeight()/(nyMax-nyMin));
    sczX -= sczX>>3; sczY -= sczY>>4;
    scX = (int)((nxMax+nxMin)/2-dim.getWidth()/2*256/sczX);
    scY = (int)((nyMax+nyMin)/2-dim.getHeight()/2*256/sczY);

    BasicStroke srk;
    Font fnt;
    FontMetrics fmtr;

    g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_OFF);
    g.setColor(trWhite);
    g.fillRect(0, 0, (int)dim.getWidth(), (int)dim.getHeight());
    g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

    srk = new BasicStroke(1);
    g.setStroke(srk);
    Enumeration elms = nodePoints.elements();
    while ( elms.hasMoreElements() ) {
      ChmapTreeNode tn = (ChmapTreeNode)elms.nextElement();
      if ( tn.x == ChmapLayouter.NOT_EXIST ) continue;
      ChmapNode node = tn.node;
      Enumeration be = node.branch.elements();
      while ( be.hasMoreElements() ) {
        ChmapTreeNode bn = ((ChmapNode)be.nextElement()).treeNode;
        if ( bn.x == ChmapLayouter.NOT_EXIST ) continue;
        int nx = (tn.x-scX)*sczX/256, ny = (tn.y-scY)*sczY/256;
        int px = (bn.x-scX)*sczX/256, py = (bn.y-scY)*sczY/256;
        g.setColor(trGreen);
        g.drawLine(px, py, nx, ny);
      }
    }

    fnt = new Font("SansSerif", Font.BOLD, 8);
    g.setFont(fnt);
    fmtr = g.getFontMetrics();

    srk = new BasicStroke(2);
    g.setStroke(srk);
    RoundRectangle2D.Float rrf = new RoundRectangle2D.Float();
    elms = nodePoints.elements();
    while ( elms.hasMoreElements() ) {
      ChmapTreeNode tn = (ChmapTreeNode)elms.nextElement();
      if ( tn.x == ChmapLayouter.NOT_EXIST ) continue;
      int nx = (tn.x-scX)*sczX/256, ny = (tn.y-scY)*sczY/256;
      if ( tn.node.thread ) {
        g.setColor(trSilver);
        g.drawString(tn.node.word, nx, ny);
      }
    }
    elms = nodePoints.elements();
    while ( elms.hasMoreElements() ) {
      ChmapTreeNode tn = (ChmapTreeNode)elms.nextElement();
      if ( tn.x == ChmapLayouter.NOT_EXIST ) continue;
      int nx = (tn.x-scX)*sczX/256, ny = (tn.y-scY)*sczY/256;
      if ( tn.node.thread ) {
        rrf.x = nx-4; rrf.y = ny-4; rrf.width = 8; rrf.height = 8;
        rrf.archeight = 3; rrf.arcwidth = 3;
        g.setColor(trBlue);
        g.fill(rrf);
      }
    }
    elms = nodePoints.elements();
    while ( elms.hasMoreElements() ) {
      ChmapTreeNode tn = (ChmapTreeNode)elms.nextElement();
      if ( tn.x == ChmapLayouter.NOT_EXIST ) continue;
      int nx = (tn.x-scX)*sczX/256, ny = (tn.y-scY)*sczY/256;
      if ( !tn.node.thread ) {
        String kwd = tn.node.word;
        if ( kwd.length() >= KEYWORD_MAX_LGT ) {
          kwd = kwd.substring(0, KEYWORD_MAX_LGT) + "..";
        }
        int width = fmtr.stringWidth(kwd);
        int height = fmtr.getHeight();
        ChmapNode cn = tn.node;
        rrf.width = width+cn.cnt/8; rrf.height = height+cn.cnt/8;
        rrf.x = nx-rrf.width/2; rrf.y = ny-rrf.height/2;
        rrf.archeight = 5; rrf.arcwidth = 2;
        g.setColor(trGreen);
        g.fill(rrf);
      }
    }

    fnt = new Font("SansSerif", Font.BOLD, 9);
    g.setFont(fnt);
    fmtr = g.getFontMetrics();

    elms = nodePoints.elements();
    while ( elms.hasMoreElements() ) {
      ChmapTreeNode tn = (ChmapTreeNode)elms.nextElement();
      if ( tn.x == ChmapLayouter.NOT_EXIST ) continue;
      int nx = (tn.x-scX)*sczX/256, ny = (tn.y-scY)*sczY/256;
      if ( !tn.node.thread ) {
        String kwd = tn.node.word;
        if ( kwd.length() >= KEYWORD_MAX_LGT ) {
          kwd = kwd.substring(0, KEYWORD_MAX_LGT) + "..";
        }
        int width = fmtr.stringWidth(kwd);
        int height = fmtr.getHeight();
        g.setColor(trBlack);
        g.drawString(kwd, nx-(int)(width*0.5), ny+(int)(height*0.4));
      }
    }
  }
}
