/*
 * $Id: ChmapLayouter.java,v 1.2 2002/08/14 15:05:34 kenta Exp $
 *
 * Copyright 2002 Kenta Cho. All rights reserved.
 */
package jp.gr.java_conf.abagames.chmap;

import java.util.*;

/**
 * Chmap graph layouter.
 *
 * @author cs8k-cyu@asahi-net.or.jp
 * @version $Revision: 1.2 $
 */
public class ChmapLayouter {

  private final int CANVAS_SIZE = 65536;

  private Random rnd = new Random();
  private Vector nodePoints;

  private void setInitialPlaces(Hashtable nodes) {
    Enumeration elms = nodes.elements();
    while ( elms.hasMoreElements() ) {
      ChmapNode cn = (ChmapNode)elms.nextElement();
      ChmapTreeNode tn = new ChmapTreeNode(cn);
      cn.treeNode = tn;
      tn.x = Math.abs(rnd.nextInt())%CANVAS_SIZE;
      tn.y = Math.abs(rnd.nextInt())%CANVAS_SIZE;
      nodePoints.addElement(tn);
    }
  }

  public static final int NOT_EXIST = -99999;

  public int xMin, xMax, yMin, yMax;
  private int cx, cy;

  private void adjustPlaces(Hashtable nodes) {
    Enumeration elms = nodes.elements();
    xMin = yMin = 99999; xMax = yMax = -99999;
    cx = cy = 0;
    int cnt = 0;
    while ( elms.hasMoreElements() ) {
      ChmapNode cn = (ChmapNode)elms.nextElement();
      ChmapTreeNode tn = cn.treeNode;
      if ( tn.x == NOT_EXIST ) continue;
      int x = 0, y = 0, c = 0;
      Enumeration be = cn.branch.elements();
      while ( be.hasMoreElements() ){
        ChmapTreeNode btn = ((ChmapNode)be.nextElement()).treeNode;
        if ( btn.x == NOT_EXIST ) continue;
        x += btn.x; y += btn.y; c++;
      }
      if ( c > 1 ) {
        tn.x += (x/c-tn.x)>>2;
        tn.y += (y/c-tn.y)>>2;
        if ( tn.x < xMin ) xMin = tn.x;
        if ( tn.x > xMax ) xMax = tn.x;
        if ( tn.y < yMin ) yMin = tn.y;
        if ( tn.y > yMax ) yMax = tn.y;
        cx += tn.x; cy += tn.y; cnt++;
      } else {
        tn.x = NOT_EXIST;
      }
    }
    cx /= cnt; cy /= cnt;
  }

  private void levelPlaces(Hashtable nodes) {
    Enumeration elms = nodes.elements();
    while ( elms.hasMoreElements() ) {
      ChmapNode cn = (ChmapNode)elms.nextElement();
      ChmapTreeNode tn = cn.treeNode;
      if ( tn.x == NOT_EXIST ) continue;
      if ( rnd.nextBoolean() ) {
        if ( tn.x < cx ) {
          tn.x -= ((tn.x-xMin)>>4);
        } else {
          tn.x += ((xMax-tn.x)>>4);
        }
      } else {
        if ( tn.y < cy ) {
          tn.y -= ((tn.y-yMin)>>4);
        } else {
          tn.y += ((yMax-tn.y)>>4);
        }
      }
    }
  }

  public Vector layoutChmapNodes(Hashtable nodes) {
    nodePoints = new Vector();
    setInitialPlaces(nodes);
    for ( int i=0 ; i<24 ; i++ ) {
      levelPlaces(nodes);
      adjustPlaces(nodes);
    }
    return nodePoints;
  }
}
