/*
 * $Id: WebPage.java,v 1.2 2002/08/14 15:05:34 kenta Exp $
 *
 * Copyright 2002 Kenta Cho. All rights reserved.
 */
package jp.gr.java_conf.abagames.chmap;

import java.io.*;
import java.util.*;
import java.net.*;

/**
 * WWW page data.
 *
 * @author cs8k-cyu@asahi-net.or.jp
 * @version $Revision: 1.2 $
 */
public class WebPage implements Runnable {

  public WebPage() { }

  public WebPage(String url)
   throws IOException, UnsupportedEncodingException {
    targetURL = url;
    getPage();
  }

  private synchronized void waitTillDown(long timeout) {
    try {
      wait(timeout);
    } catch ( InterruptedException e ) {}
  }

  private synchronized final void notifyDown() {
    try {
      notifyAll();
    } catch ( Exception e ) {}
  }

  private boolean downloaded;

  private String targetURL;
  private String contents;

  private int timeout = 30000;

  public void run() {
    StringBuffer buffer = new StringBuffer();
    try {
      URL url;
      url = new URL(targetURL);
      URLConnection cnn = url.openConnection();
      BufferedReader reader = new BufferedReader(
       new InputStreamReader(cnn.getInputStream(), "JISAutoDetect"));
      for ( ; ; ) {
        String line = reader.readLine();
        if ( line == null ) break;
        buffer.append(line);
      }
      contents = buffer.toString();
    } catch ( IOException e ) {
      e.printStackTrace();
      contents = null;
    }
    downloaded = true;
    notifyDown();
  }

  private void getPage()
   throws IOException, UnsupportedEncodingException {
    downloaded = false;
    new Thread(this).start();
    waitTillDown(timeout);
    if ( !downloaded ) {
      throw new IOException("time out.");
    }
  }

  public String getContents() {
    return contents;
  }

  public void setContents(String contents) {
    this.contents = contents;
  }

  public String getTitle() {
    int titleIdx = contents.indexOf("<TITLE>");
    if ( titleIdx != -1 ) {
      String titleSbst = contents.substring(titleIdx+7);
      int titleEndIdx = titleSbst.indexOf("</TITLE>");
      if ( titleEndIdx != -1 ) {
        return titleSbst.substring(0, titleEndIdx);
      }
    }
    titleIdx = contents.indexOf("<title>");
    if ( titleIdx != -1 ) {
      String titleSbst = contents.substring(titleIdx+7);
      int titleEndIdx = titleSbst.indexOf("</title>");
      if ( titleEndIdx != -1 ) {
        return titleSbst.substring(0, titleEndIdx);
      }
    }
    return "";
  }

  public String getBase() {
    int baseIdx = contents.indexOf("<base href=\"");
    if ( baseIdx != -1 ) {
      int baseEndIdx = contents.indexOf("\"", baseIdx+12);
      if ( baseEndIdx != -1 ) {
        return contents.substring(baseIdx+12, baseEndIdx);
      }
    }
    return "";
  }

  public Vector getAnchors() {
    Vector anchors = new Vector();
    int idx = 0;
    for ( ; ; ) {
      boolean dblq = false;
      int ancIdx = contents.indexOf("<a href=", idx);
      if ( ancIdx == -1 ) break;
      if ( contents.charAt(ancIdx+8) == '\"' ) {
        ancIdx++;
        dblq = true;
      }
      int ancEndIdx;
      if ( dblq ) {
        ancEndIdx = contents.indexOf("\">", ancIdx);
      } else {
        ancEndIdx = contents.indexOf(">", ancIdx);
      }
      if ( ancEndIdx == -1 ) break;
      anchors.addElement(contents.substring(ancIdx+8, ancEndIdx));
      idx = ancEndIdx;
    }
    return anchors;
  }

  public Vector getRess() {
    Vector ress = new Vector();
    int idx = 0;
    for ( ; ; ) {
      int rsIdx = contents.indexOf("<dd>", idx);
      if ( rsIdx == -1 ) break;
      int rsEndIdx = contents.indexOf("<dt>", rsIdx+4);
      if ( rsEndIdx == -1 ) break;
      String rs = contents.substring(rsIdx+4, rsEndIdx);
      StringBuffer rsb = new StringBuffer();
      int brIdx = 0;
      for ( ; ; ) {
        int brsIdx = rs.indexOf("<br>", brIdx);
        if ( brsIdx == -1 ) {
          rsb.append(rs.substring(brIdx));
          break;
        }
        rsb.append(rs.substring(brIdx, brsIdx));
        brIdx = brsIdx+4;
      }
      rs = rsb.toString();
      rsb = new StringBuffer();
      int ancIdx = 0;
      for ( ; ; ) {
        int ancsIdx = rs.indexOf("<a", ancIdx);
        if ( ancsIdx == -1 ) {
          rsb.append(rs.substring(ancIdx));
          break;
        }
        rsb.append(rs.substring(ancIdx, ancsIdx));
        ancIdx = rs.indexOf("a>", ancsIdx+2) + 2;
      }
      rs = rsb.toString();
      rsb = new StringBuffer();
      int ampIdx = 0;
      for ( ; ; ) {
        int ampsIdx = rs.indexOf("&", ampIdx);
        if ( ampsIdx == -1 ) {
          rsb.append(rs.substring(ampIdx));
          break;
        }
        rsb.append(rs.substring(ampIdx, ampsIdx));
        int ampeIdx = rs.indexOf(";", ampsIdx+1);
        if ( ampeIdx == -1 ) {
          rsb.append(rs.substring(ampsIdx));
          break;
        }
        ampIdx = ampeIdx + 1;
      }
      ress.addElement(rsb.toString());
      idx = rsEndIdx;
    }
    return ress;
  }

  public boolean removeTillDd() {
    return removeTill("<dd>");
  }

  //**

  private String removeAmp(String rs) {
    StringBuffer rsb = new StringBuffer();
    int ampIdx = 0;
    for ( ; ; ) {
      int ampsIdx = rs.indexOf("&", ampIdx);
      if ( ampsIdx == -1 ) {
        rsb.append(rs.substring(ampIdx));
        break;
      }
      rsb.append(rs.substring(ampIdx, ampsIdx));
      int ampeIdx = rs.indexOf(";", ampsIdx+1);
      if ( ampeIdx == -1 ) {
        rsb.append(rs.substring(ampsIdx));
        break;
      }
      ampIdx = ampeIdx + 1;
    }
    return rsb.toString();
  }

  public Vector getContentsPieces() {
    Vector pces = new Vector();
    StringBuffer tdc = new StringBuffer();
    int idx = 0, pIdx;
    for ( ; ; ) {
      pIdx = idx;
      idx = contents.indexOf('<', idx);
      if ( idx == -1 ) {
        String pce = removeAmp(contents.substring(pIdx));
        pces.addElement(pce);
        break;
      }
      String pce = removeAmp(contents.substring(pIdx, idx));
      pces.addElement(pce);
      idx++;
      idx = contents.indexOf('>', idx);
      if ( idx == -1 ) break;
      idx++;
    }
    return pces;
  }

  public boolean removeTill(String str) {
    int rsIdx = contents.indexOf(str, 1);
    if ( rsIdx == -1 ) return false;
    contents = contents.substring(rsIdx);
    return true;
  }
}
